
use <pencil-holder.scad>;

m3_nut_width = 5.5;
m5_nut_width = 7.9;
backlash = 0.25;
shaft_length = 150;
show_pencils = "no";

module baseGrooveProfile() {
    difference() {
        translate([25, 0])
            polygon([
                [-9, 0],
                [-9, 2],
                [-7, 4.5],
                [8.5, 4.5],
                [8.5, 10.5],
                [15, 10.5],
                [9, 0],
            ]);
        translate([25, 6]) scale([1.07, 1]) circle(r=5, $fn=100);
    }
}

module base() {
    rotate_extrude($fn=200) baseGrooveProfile();
}

module bearingsDev() {
    for (i=[0:15]) {
        rotate([0, 0, i * 21.9])
            translate([25, 0, 9.5 / 2 + 1])
                color([0.6, 0.6, 0.6])
                    sphere(d=9.5, $fn=50);
    }
}

module floaterGrooveProfile() {
    translate([0, 11.5]) scale([1, -1]) difference() {
        translate([25, 0])
            polygon([
                [-7, 0],
                [-7, 4.5],
                [7, 4.5],
                [7, 0],
            ]);
        translate([25, 6]) scale([1.07, 1]) circle(r=5, $fn=100);
    }
    //color([1, 0, 0]) translate([10.3, 11.4]) square([35, 2]);
    polygon([
        [8, 13.4],
        [48, 13.4],
        [48, 10],
        [41, 10],
        [41, 11.4],
        [15, 11.4],
        [12, 6],
        [8, 6],
    ]);
}

module platformProfile() {
    polygon([
        [41, 10],
        [48, 10],
        [48, 2],
        [36.3, 2],
    ]);
};

module bearingProfileDev() {
    translate([25, 9.5 / 2 + 1]) circle(d=9.5, $fn=100);
}

module baseProfileDev() {
    floaterGrooveProfile();
    baseGrooveProfile();
    platformProfile();
    bearingProfileDev();
}

//baseProfileDev();

module floaterScrewHoles() {
    for (i=[0:5]) {
        rotate([0, 0, 60 * i]) {
            translate([44, 0, 5.5])
                cylinder(r=1.5 + backlash, h=15, $fn=19);
            translate([44, 0, 1.9])
                rotate([0, 0, 30])
                    cylinder(d=m3_nut_width / 0.87 + backlash * 2, h=3, $fn=6);
        }
    }
}

module floater() {
    difference() {
        rotate_extrude($fn=200) floaterGrooveProfile();
        cylinder(r=10.4, h=20, $fn=6);
        floaterScrewHoles();
    }
}

module shaftShelf() {
    intersection() {
        cylinder(r=16, h=3, $fn=6);
        translate([0, -11.34, 0]) cube(40, center=true);
    }
}

module shaftScrewHole() {
    rotate([0, 0, -90]) {
        cylinder(d=5 + backlash * 2, h=40, $fn=19);
        nut_width = m5_nut_width + backlash * 2;
        translate([0, 0, 8]) cylinder(d=nut_width / 0.87, h=5.3, $fn=6);
        translate([1, -nut_width / 2, 8]) cube([15, nut_width, 5.3]);
    }
}

module halfShaft() {
    difference() {
        union() {
            translate([0, 0, 6]) cylinder(r=10, h=shaft_length / 2, $fn=6);
            translate([0, 0, 13.4]) shaftShelf();
        }
        shaftScrewHole();
    }
}

module shaft() {
    halfShaft();
    translate([0, 0, 12 + shaft_length]) scale([1, 1, -1]) halfShaft();
}

module platform() {
    difference() {
        rotate_extrude($fn=200) platformProfile();
        floaterScrewHoles();
    }
    translate([0, 0, 2]) {
        pencil_holder(show_pencils=show_pencils);
        for (i=[0:11]) {
            rotate([0, 0, 30 * i + 15]) translate([47, -3, 0]) cube([13, 6, 6]);
        }
    }
}

//platform();

module bottomCap() {
    difference() {
        translate([0, 0, 5.25]) cylinder(r=12, h=0.75, $fn=37);
        cylinder(d=5 + backlash * 2, h=20, $fn=37);
    }
    difference() {
        translate([0, 0, 3.5]) cylinder(r=12, h=2.1, $fn=37);
        cylinder(r=8, h=10, $fn=37);
    }
}

module basket() {
    translate([0, 0, 6 + shaft_length - 7.4]) {
        difference() {
            union() {
                cylinder(r=45, h=3, $fn=120);
                cylinder(r=12, h=7.4, $fn=120);
            }
            translate([0, 0, -0.1]) {
                cylinder(r=10.4, h=10, $fn=6);
            }
        }
        difference() {
            cylinder(r1=45, r2=55, h=20, $fn=120);
            translate([0, 0, -0.1]) cylinder(r1=42, r2=52, h=21, $fn=120);
        }
    }
}

module topCap() {
    translate([0, 0, 6 + shaft_length])
        difference() {
            intersection() {
                translate([0, 0, 16])
                    scale([0.8, 0.8, 1])
                        sphere(r=22, $fn=100);
                translate([0, 0, 25])
                    cube(50, center=true);
            }
            translate([0, 0, -0.1])
                cylinder(d=5 + backlash * 2, h=50, $fn=37);
            translate([0, 0, 2])
                cylinder(r=7, h=50, $fn=37);
        }
}

module crossSectionDev() {
    intersection() {
        union() {
            base();
            bearingsDev();
            floater();
            shaft();
            platform(show_pencils="yes");
            bottomCap();
            basket();
            topCap();
        }
        cube(300);
    }
}

//crossSectionDev();

module printableBase() {
    base();
}

module printableFloater() {
    translate([0, 0, 13.4]) rotate([0, 180, 0]) floater();
}

module printableShaft() {
    translate([0, 0, 8.66]) rotate([-90, 0, 0]) shaft();
}

module printablePlatform() {
    translate([0, 0, -2]) platform();
}

module printableBottomCap() {
    translate([0, 0, 6]) rotate([0, 180, 0]) bottomCap();
}

module printableBasket() {
    translate([0, 0, -shaft_length - 6 + 7.4]) basket();
}

module printableTopCap() {
    translate([0, 0, -shaft_length - 6]) topCap();
}

//printableBase();
//printableFloater();
//printableShaft();
printablePlatform();
//printableBottomCap();
//printableBasket();
//printableTopCap();

